#! /usr/bin/perl
#
use strict;
use warnings;

use Test::More tests => 1;
use CracTools::Annotator;
use Inline::Files 0.68;
use File::Temp;

# Create a temp file with the GFF lines described below
my $gff_file = new File::Temp( SUFFIX => '.gff', UNLINK => 1);
while(<GFF>) {print $gff_file $_;}
close $gff_file;

#my $annotator = CracTools::Annotator->new($gff_file);
my $annotator = CracTools::Annotator->new($gff_file,"fast");

ok(1);

__GFF__
1	Ensembl_CORE	exon	12	42	.	+	.	ID=ENSE00002706393;Parent=ENST00000578939
1	Ensembl_CORE	exon	72	102	.	+	.	ID=ENSE00002706394;Parent=ENST00000578939,ENST00000578940
1	Ensembl_CORE	cds	12	102	.	+	.	ID=ENST00000578939.cds;Parent=ENST00000578939
